#!/usr/bin/perl -wT
#
# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is the Bugzilla Bug Tracking System.
# 
# The Initial Developer of the Original Code is Netscape Communications
# Corporation. Portions created by Netscape are Copyright (C) 1998
# Netscape Communications Corporation. All Rights Reserved.
#
# Contributor(s): Alexey Gladkov <legion@altlinux.org>

use diagnostics;
use strict;
use lib ".";

require "CGI.pl";

use Bugzilla::Constants;

use vars qw(
            @legal_bug_status
            %FORM
			%versions
            $template
            $vars
            );

#ConnectToDatabase();

my $cgi = Bugzilla->cgi;

if (defined $::FORM{"GoAheadAndLogIn"}) {
    # We got here from a login page, probably from relogin.cgi.  We better
    # make sure the password is legit.
    Bugzilla->login(LOGIN_REQUIRED);
} else {
    Bugzilla->login();
}

my $userid = Bugzilla->user->id;

GetVersionTable(); 

my @products = ();
foreach my $p (GetSelectableProducts()) {
    push @products,$p;
}
@products = sort { lc($a) cmp lc($b) } @products;

# Create data structures representing each product.
for (my $i = 0; $i < @products; ++$i) {
    my $p = $products[$i];
    
    # Create hash to hold attributes for each product.
    my %product = (
        'name'       => $p,
        'versions'   => [ sort { lc($a) cmp lc($b) } @{$::versions{$p}}   ]
    );
    
    # Assign hash back to product array.
    $products[$i] = \%product;
}

if (@products) {
    $vars->{'product'} = \@products;
    $vars->{'bug_status'} = \@::legal_bug_status;
    $vars->{'format'} = $::FORM{'format'};
    $vars->{'query_format'} = $::FORM{'query_format'};

    my $format = GetFormat("search/search", 
                       $::FORM{'query_format'} || $::FORM{'format'} || 'easy', 
                       $::FORM{'ctype'});

    print $cgi->header($format->{'ctype'});
    $template->process($format->{'template'}, $vars)
	|| ThrowTemplateError($template->error());
} else {
    ThrowUserError("invalid_product_name");
}
